use log::debug;
use std::sync::Arc;

use serenity::{
    async_trait,
    model::prelude::interaction::{
        application_command::ApplicationCommandInteraction, InteractionResponseType,
    },
    prelude::Context,
};

use crate::{
    database::Client as DatabaseClient,
    discord::message_builders::embed_builder::EmbedMessageBuilder,
};

use super::commons::{BotCommand, BotCommandOption, Error};

pub struct SourceCodeCommand {
    interaction: ApplicationCommandInteraction,
}

#[async_trait]
impl BotCommand for SourceCodeCommand {
    fn name() -> String {
        String::from("about")
    }

    fn description() -> String {
        String::from("Display the bot credentials")
    }

    fn options_list() -> Vec<BotCommandOption> {
        vec![]
    }

    fn new(interaction: ApplicationCommandInteraction) -> Self {
        debug!("Creating a new SourceCodeCommand object");
        SourceCodeCommand { interaction }
    }

    async fn run(&self, context: Context, _: Arc<DatabaseClient>) -> Result<(), Error> {
        let embed_builder = EmbedMessageBuilder::new(&context).await?;
        self.interaction
            .create_interaction_response(context.http, |response| {
                response
                    .kind(InteractionResponseType::ChannelMessageWithSource)
                    .interaction_response_data(|r| {
                        r.set_embed(embed_builder.create_bot_credentials_embed())
                    })
            })
            .await
            .map_err(Error::DiscordAPICall)
    }
}
